import cupy as cp
import numpy as np

with open('filtering.cu') as f:
    code = f.read()

kers = ('exact_one', 'filter')
ep_pontok_module = cp.RawModule(code=code, options=('--std=c++11',), name_expressions=kers)
exact_one_cuda = ep_pontok_module.get_function(kers[0])
filter_cuda = ep_pontok_module.get_function(kers[1])

def expSpace(min, max, N, exponentialliness = 20.0):
    LinVec = cp.linspace(0, cp.log10(exponentialliness+1, dtype=cp.float64),N, dtype=cp.float64)
    return (max-min)/exponentialliness * (10.0**LinVec - 1) + min


def convert(egyensulyi_mtx):
    parok = np.empty([0], dtype=np.int8)

    for i in egyensulyi_mtx:
        for S in range(0, 4):
            for U in range(0, 4):
                if i[S][U] == 1:
                    parok = np.append(parok, S+1)
                    parok = np.append(parok, U+1)
        parok = np.append(parok, 0)
        parok = np.append(parok, 0)
    
    N = parok.size
    return np.resize(parok, (int(N/2), 2))

def printresults(egyensulyi_mtx):
    ossz = 0
    for i in egyensulyi_mtx:
        parok = np.empty([0], dtype=np.int8)
        for S in range(0, 4):
            for U in range(0, 4):
                if i[S][U] == 1:
                    parok = np.append(parok, S+1)
                    parok = np.append(parok, U+1)
        N = parok.size
        print(f"{int(N/2)}x2")
        print(np.resize(parok, (int(N/2), 2)))
        print()

def search(egyensulyi_mtx, S, U):
    for i in egyensulyi_mtx:
        if i[S][U] == 1:
            print(i)

def exact_one(egyensulyi_mtx, S, U):
    for i in egyensulyi_mtx:
        ok = True
        for j in range(0, 4):
            for k in range(0, 4):
                if j == S and k == U and i[j][k] == 0:
                    ok = False
                if j != S and k != U and i[j][k] == 1:
                    ok = False
                if j == S and k != U and i[j][k] == 1:
                    ok = False
                if j != S and k == U and i[j][k] == 1:
                    ok = False
        if ok:
            print(i)

def exact_one_gpu(egyensulyi_mtx, S, U):
    size = int(egyensulyi_mtx.size / 16)
    indexes = cp.zeros((size,), dtype=cp.bool)
    numBlock = int((size + 256 - 1) / 256)
    exact_one_cuda((numBlock,), (256,), (egyensulyi_mtx, size, indexes, S, U))
    return egyensulyi_mtx[indexes]


def filter_gpu(egyensulyi_mtx, S, U):
    size = int(egyensulyi_mtx.size / 16)
    indexes = cp.zeros((size,), dtype=cp.bool)
    numBlock = int((size + 256 - 1) / 256)
    filter_cuda((numBlock,), (256,), (egyensulyi_mtx, size, indexes, S, U))
    return egyensulyi_mtx[indexes]

def compute_lcm(x, y):
   if x > y:
       greater = x
   else:
       greater = y
   while(True):
       if((greater % x == 0) and (greater % y == 0)):
           lcm = greater
           break
       greater += 1
   return lcm